@@ -142,20 +142,20 @@ lws_daemonize(const char *_lock_path)
 		exit(9);
 	}
 
-	/* If we got a good PID, then we can exit the parent process. */
+        /* If we got a good PID, then we can exit the parent process. */
 	if (pid_daemon > 0) {
 
-		/*
-		 * Wait for confirmation signal from the child via
-		 * SIGCHILD / USR1, or for two seconds to elapse
-		 * (SIGALRM).  pause() should not return.
-		 */
-		alarm(2);
+               /*
+                * Wait for confirmation signal from the child via
+                * SIGCHILD / USR1, or for two seconds to elapse
+                * (SIGALRM).  pause() should not return.
+                */
+               alarm(2);
 
-		pause();
-		/* should not be reachable */
-		exit(0);
-	}
+               pause();
+               /* should not be reachable */
+               exit(1);
+       }
 
 	/* At this point we are executing as the child process */
 	parent = getppid();
@@ -184,7 +184,7 @@ lws_daemonize(const char *_lock_path)
 	 * Change the current working directory.  This prevents the current
 	 * directory from being locked; hence not being able to remove it.
 	 */
-	if (chdir("/") < 0) {
+	if (chdir("/tmp") < 0) {
 		fprintf(stderr,
 			"unable to change directory to %s, code %d (%s)",
 			"/", errno, strerror(errno));
