@@ -145,8 +145,15 @@ lws_add_http_header_status(struct lws *wsi, unsigned int code,
 			   unsigned char **p, unsigned char *end)
 {
 	unsigned char code_and_desc[60];
-	const char *description = "";
+	const char *description = "", *p1;
 	int n;
+	static const char * const hver[] = {
+		"HTTP/1.0", "HTTP/1.1", "HTTP/2"
+	};
+
+#ifdef LWS_WITH_ACCESS_LOG
+	wsi->access_log.response = code;
+#endif
 
 #ifdef LWS_USE_HTTP2
 	if (wsi->mode == LWSCM_HTTP2_SERVING)
@@ -157,10 +164,42 @@ lws_add_http_header_status(struct lws *wsi, unsigned int code,
 	if (code >= 500 && code < (500 + ARRAY_SIZE(err500)))
 		description = err500[code - 500];
 
-	n = sprintf((char *)code_and_desc, "HTTP/1.0 %u %s", code, description);
+	if (code == 200)
+		description = "OK";
 
-	return lws_add_http_header_by_name(wsi, NULL, code_and_desc,
-					   n, p, end);
+	if (code == 304)
+		description = "Not Modified";
+	else
+		if (code >= 300 && code < 400)
+			description = "Redirect";
+
+	if (wsi->u.http.request_version < ARRAY_SIZE(hver))
+		p1 = hver[wsi->u.http.request_version];
+	else
+		p1 = hver[0];
+
+	n = sprintf((char *)code_and_desc, "%s %u %s",
+		    p1, code, description);
+
+	if (lws_add_http_header_by_name(wsi, NULL, code_and_desc,
+					   n, p, end))
+		return 1;
+
+	if (lws_add_http_header_by_token(wsi, WSI_TOKEN_HTTP_SERVER,
+					 (unsigned char *)
+					 	 wsi->context->server_string,
+					 wsi->context->server_string_len,
+					 p, end))
+		return 1;
+
+	if (wsi->vhost->options & LWS_SERVER_OPTION_STS)
+		if (lws_add_http_header_by_name(wsi, (unsigned char *)
+				"Strict-Transport-Security:",
+				(unsigned char *)"max-age=15768000 ; "
+				"includeSubDomains", 36, p, end))
+			return 1;
+
+	return 0;
 }
 
 /**
@@ -192,10 +231,7 @@ lws_return_http_status(struct lws *wsi, unsigned int code,
 
 	if (lws_add_http_header_status(wsi, code, &p, end))
 		return 1;
-	if (lws_add_http_header_by_token(wsi, WSI_TOKEN_HTTP_SERVER,
-					 (unsigned char *)"libwebsockets", 13,
-					 &p, end))
-		return 1;
+
 	if (lws_add_http_header_by_token(wsi, WSI_TOKEN_HTTP_CONTENT_TYPE,
 					 (unsigned char *)"text/html", 9,
 					 &p, end))
@@ -217,3 +253,42 @@ lws_return_http_status(struct lws *wsi, unsigned int code,
 
 	return m != n;
 }
+
+LWS_VISIBLE int
+lws_http_redirect(struct lws *wsi, int code, const unsigned char *loc, int len,
+		  unsigned char **p, unsigned char *end)
+{
+	unsigned char *start = *p;
+	int n;
+
+	if (lws_add_http_header_status(wsi, code, p, end))
+		return -1;
+
+	if (lws_add_http_header_by_token(wsi,
+			WSI_TOKEN_HTTP_LOCATION,
+			loc, len, p, end))
+		return -1;
+	/*
+	 * if we're going with http/1.1 and keepalive,
+	 * we have to give fake content metadata so the
+	 * client knows we completed the transaction and
+	 * it can do the redirect...
+	 */
+	if (lws_add_http_header_by_token(wsi,
+			WSI_TOKEN_HTTP_CONTENT_TYPE,
+			(unsigned char *)"text/html", 9,
+			p, end))
+		return -1;
+	if (lws_add_http_header_by_token(wsi,
+			WSI_TOKEN_HTTP_CONTENT_LENGTH,
+			(unsigned char *)"0", 1, p, end))
+		return -1;
+
+	if (lws_finalize_http_header(wsi, p, end))
+		return -1;
+
+	n = lws_write(wsi, start, *p - start,
+			LWS_WRITE_HTTP_HEADERS);
+
+	return n;
+}
