@@ -52,10 +52,10 @@ lws_http2_wsi_from_id(struct lws *wsi, unsigned int sid)
 }
 
 struct lws *
-lws_create_server_child_wsi(struct lws_context *context, struct lws *parent_wsi,
+lws_create_server_child_wsi(struct lws_vhost *vhost, struct lws *parent_wsi,
 			    unsigned int sid)
 {
-	struct lws *wsi = lws_create_new_server_wsi(context);
+	struct lws *wsi = lws_create_new_server_wsi(vhost);
 
 	if (!wsi)
 		return NULL;
@@ -64,8 +64,7 @@ lws_create_server_child_wsi(struct lws_context *context, struct lws *parent_wsi,
 	if (parent_wsi->u.http2.child_count + 1 ==
 	    parent_wsi->u.http2.peer_settings.setting[
 			LWS_HTTP2_SETTINGS__MAX_CONCURRENT_STREAMS])
-		return NULL;
-
+		goto bail;
 	lws_http2_init(&wsi->u.http2.peer_settings);
 	lws_http2_init(&wsi->u.http2.my_settings);
 	wsi->u.http2.stream_id = sid;
@@ -82,13 +81,21 @@ lws_create_server_child_wsi(struct lws_context *context, struct lws *parent_wsi,
 	wsi->state = LWSS_HTTP2_ESTABLISHED;
 	wsi->mode = parent_wsi->mode;
 
-	wsi->protocol = &context->protocols[0];
-	lws_ensure_user_space(wsi);
+	wsi->protocol = &vhost->protocols[0];
+	if (lws_ensure_user_space(wsi))
+		goto bail;
 
 	lwsl_info("%s: %p new child %p, sid %d, user_space=%p\n", __func__,
 		  parent_wsi, wsi, sid, wsi->user_space);
 
 	return wsi;
+
+bail:
+	vhost->protocols[0].callback(wsi, LWS_CALLBACK_WSI_DESTROY,
+			       NULL, NULL, 0);
+	lws_free(wsi);
+
+	return NULL;
 }
 
 int lws_remove_server_child_wsi(struct lws_context *context, struct lws *wsi)
@@ -198,7 +205,6 @@ static const char * https_client_preface =
 int
 lws_http2_parser(struct lws *wsi, unsigned char c)
 {
-	struct lws_context *context = wsi->context;
 	struct lws *swsi;
 	int n;
 
@@ -243,8 +249,15 @@ lws_http2_parser(struct lws *wsi, unsigned char c)
 			case LWS_HTTP2_FRAME_TYPE_CONTINUATION:
 			case LWS_HTTP2_FRAME_TYPE_HEADERS:
 				lwsl_info(" %02X\n", c);
-				if (lws_hpack_interpret(wsi->u.http2.stream_wsi, c))
+				if (!wsi->u.http2.stream_wsi->u.hdr.ah)
+					if (lws_header_table_attach(wsi->u.http2.stream_wsi, 0)) {
+						lwsl_err("%s: Failed to get ah\n", __func__);
+						return 1;
+					}
+				if (lws_hpack_interpret(wsi->u.http2.stream_wsi, c)) {
+					lwsl_notice("%s: lws_hpack_interpret failed\n", __func__);
 					return 1;
+				}
 				break;
 			case LWS_HTTP2_FRAME_TYPE_GOAWAY:
 				if (wsi->u.http2.count >= 5 && wsi->u.http2.count <= 8) {
@@ -377,8 +390,11 @@ lws_http2_parser(struct lws *wsi, unsigned char c)
 				lwsl_info("LWS_HTTP2_FRAME_TYPE_HEADERS: stream_id = %d\n", wsi->u.http2.stream_id);
 				if (!wsi->u.http2.stream_id)
 					return 1;
-				if (!wsi->u.http2.stream_wsi)
-					wsi->u.http2.stream_wsi = lws_create_server_child_wsi(context, wsi, wsi->u.http2.stream_id);
+				if (!wsi->u.http2.stream_wsi) {
+					wsi->u.http2.stream_wsi =
+						lws_create_server_child_wsi(wsi->vhost, wsi, wsi->u.http2.stream_id);
+					wsi->u.http2.stream_wsi->http2_substream = 1;
+				}
 
 				/* END_STREAM means after servicing this, close the stream */
 				wsi->u.http2.END_STREAM = !!(wsi->u.http2.flags & LWS_HTTP2_FLAG_END_STREAM);
@@ -474,7 +490,8 @@ int lws_http2_do_pps_send(struct lws_context *context, struct lws *wsi)
 			 */
 			lwsl_info("%s: setting up sid 1\n", __func__);
 
-			swsi = wsi->u.http2.stream_wsi = lws_create_server_child_wsi(context, wsi, 1);
+			swsi = wsi->u.http2.stream_wsi =
+					lws_create_server_child_wsi(wsi->vhost, wsi, 1);
 			/* pass on the initial headers to SID 1 */
 			swsi->u.http.ah = wsi->u.http.ah;
 			wsi->u.http.ah = NULL;
