@@ -112,11 +112,20 @@ struct sockaddr_in;
 #include <winsock2.h>
 #include <ws2tcpip.h>
 #include <stddef.h>
-#include <stdint.h>
 #include <basetsd.h>
+#ifndef _WIN32_WCE
 #include <fcntl.h>
+#else
+#define _O_RDONLY	0x0000
+#define O_RDONLY	_O_RDONLY
+#endif
 
+// Visual studio older than 2015 and WIN_CE has only _stricmp
+#if (defined(_MSC_VER) && _MSC_VER < 1900) || defined(_WIN32_WCE)
+#define strcasecmp _stricmp
+#else
 #define strcasecmp stricmp
+#endif
 #define getdtablesize() 30000
 
 #define LWS_INLINE __inline
@@ -137,10 +146,18 @@ struct sockaddr_in;
 #define LWS_INVALID_FILE INVALID_HANDLE_VALUE
 #define LWS_O_RDONLY _O_RDONLY
 
+#if !defined(_MSC_VER) || _MSC_VER < 1900 /* Visual Studio 2015 already defines this in <stdio.h> */
+#define lws_snprintf _snprintf
+#endif
+
+#ifndef __func__
+#define __func__ __FUNCTION__
+#endif
+
 #else /* NOT WIN32 */
 #include <unistd.h>
 
-#if defined(__NetBSD__)
+#if defined(__NetBSD__) || defined(__FreeBSD__)
 #include <netinet/in.h>
 #endif
 
@@ -157,8 +174,15 @@ struct sockaddr_in;
 #endif
 
 #if defined(__GNUC__)
+
+/* warn_unused_result attribute only supported by GCC 3.4 or later */
+#if __GNUC__ >= 4 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4)
+#define LWS_WARN_UNUSED_RESULT __attribute__((warn_unused_result))
+#else
+#define LWS_WARN_UNUSED_RESULT
+#endif
+
 #define LWS_VISIBLE __attribute__((visibility("default")))
-#define LWS_WARN_UNUSED_RESULT __attribute__ ((warn_unused_result))
 #define LWS_WARN_DEPRECATED __attribute__ ((deprecated))
 #else
 #define LWS_VISIBLE
@@ -192,17 +216,37 @@ struct sockaddr_in;
 #endif
 
 #ifdef LWS_OPENSSL_SUPPORT
+
 #ifdef USE_WOLFSSL
 #ifdef USE_OLD_CYASSL
 #include <cyassl/openssl/ssl.h>
+#include <cyassl/error-ssl.h>
 #else
 #include <wolfssl/openssl/ssl.h>
+#include <wolfssl/error-ssl.h>
 #endif /* not USE_OLD_CYASSL */
 #else
+#if defined(LWS_USE_POLARSSL)
+#include <polarssl/ssl.h>
+struct lws_polarssl_context {
+	x509_crt ca;
+	x509_crt certificate;
+	rsa_context key;
+};
+typedef struct lws_polarssl_context SSL_CTX;
+typedef ssl_context SSL;
+#else
+#if defined(LWS_USE_MBEDTLS)
+#include <mbedtls/ssl.h>
+#else
 #include <openssl/ssl.h>
+#include <openssl/err.h>
+#endif /* not USE_MBEDTLS */
+#endif /* not USE_POLARSSL */
 #endif /* not USE_WOLFSSL */
 #endif
 
+
 #define CONTEXT_PORT_NO_LISTEN -1
 
 enum lws_log_levels {
@@ -222,6 +266,8 @@ enum lws_log_levels {
 
 LWS_VISIBLE LWS_EXTERN void _lws_log(int filter, const char *format, ...);
 LWS_VISIBLE LWS_EXTERN void _lws_logv(int filter, const char *format, va_list vl);
+LWS_VISIBLE LWS_EXTERN int
+lwsl_timestamp(int level, char *p, int len);
 
 /* notice, warn and log are always compiled in */
 #define lwsl_notice(...) _lws_log(LLL_NOTICE, __VA_ARGS__)
@@ -255,8 +301,18 @@ LWS_VISIBLE LWS_EXTERN void lwsl_hexdump(void *buf, size_t len);
 #define lwsl_hexdump(a, b)
 
 #endif
+
+#include <stddef.h>
+
+#ifndef lws_container_of
+#define lws_container_of(P,T,M)	((T *)((char *)(P) - offsetof(T, M)))
+#endif
+
+
 struct lws;
+#ifndef ARRAY_SIZE
 #define ARRAY_SIZE(x) (sizeof(x) / sizeof(x[0]))
+#endif
 
 /* api change list for user code to test against */
 
@@ -279,20 +335,32 @@ struct lws;
  * add it at where specified so existing users are unaffected.
  */
 enum lws_context_options {
-	LWS_SERVER_OPTION_REQUIRE_VALID_OPENSSL_CLIENT_CERT	= (1 << 1),
+	LWS_SERVER_OPTION_REQUIRE_VALID_OPENSSL_CLIENT_CERT	= (1 << 1) |
+								  (1 << 12),
 	LWS_SERVER_OPTION_SKIP_SERVER_CANONICAL_NAME		= (1 << 2),
-	LWS_SERVER_OPTION_ALLOW_NON_SSL_ON_SSL_PORT		= (1 << 3),
+	LWS_SERVER_OPTION_ALLOW_NON_SSL_ON_SSL_PORT		= (1 << 3) |
+								  (1 << 12),
 	LWS_SERVER_OPTION_LIBEV					= (1 << 4),
 	LWS_SERVER_OPTION_DISABLE_IPV6				= (1 << 5),
 	LWS_SERVER_OPTION_DISABLE_OS_CA_CERTS			= (1 << 6),
 	LWS_SERVER_OPTION_PEER_CERT_NOT_REQUIRED		= (1 << 7),
 	LWS_SERVER_OPTION_VALIDATE_UTF8				= (1 << 8),
-	LWS_SERVER_OPTION_SSL_ECDH				= (1 << 9),
+	LWS_SERVER_OPTION_SSL_ECDH				= (1 << 9) |
+								  (1 << 12),
 	LWS_SERVER_OPTION_LIBUV					= (1 << 10),
+	LWS_SERVER_OPTION_REDIRECT_HTTP_TO_HTTPS		= (1 << 11) |
+								  (1 << 3) |
+								  (1 << 12),
+	LWS_SERVER_OPTION_DO_SSL_GLOBAL_INIT			= (1 << 12),
+	LWS_SERVER_OPTION_EXPLICIT_VHOSTS			= (1 << 13),
+	LWS_SERVER_OPTION_UNIX_SOCK				= (1 << 14),
+	LWS_SERVER_OPTION_STS					= (1 << 15),
 
 	/****** add new things just above ---^ ******/
 };
 
+#define lws_check_opt(c, f) (((c) & (f)) == (f))
+
 /*
  * NOTE: These public enums are part of the abi.  If you want to add one,
  * add it at where specified so existing users are unaffected.
@@ -343,6 +411,16 @@ enum lws_callback_reasons {
 
 	LWS_CALLBACK_WS_EXT_DEFAULTS				= 39,
 
+	LWS_CALLBACK_CGI					= 40,
+	LWS_CALLBACK_CGI_TERMINATED				= 41,
+	LWS_CALLBACK_CGI_STDIN_DATA				= 42,
+	LWS_CALLBACK_CGI_STDIN_COMPLETED			= 43,
+	LWS_CALLBACK_ESTABLISHED_CLIENT_HTTP			= 44,
+	LWS_CALLBACK_CLOSED_CLIENT_HTTP				= 45,
+	LWS_CALLBACK_RECEIVE_CLIENT_HTTP			= 46,
+	LWS_CALLBACK_COMPLETED_CLIENT_HTTP			= 47,
+	LWS_CALLBACK_RECEIVE_CLIENT_HTTP_READ			= 48,
+
 	/****** add new things just above ---^ ******/
 
 	LWS_CALLBACK_USER = 1000, /* user code can use any including / above */
@@ -461,6 +539,7 @@ enum lws_extension_callback_reasons {
 	LWS_EXT_CB_OPTION_DEFAULT			= 23,
 	LWS_EXT_CB_OPTION_SET				= 24,
 	LWS_EXT_CB_OPTION_CONFIRM			= 25,
+	LWS_EXT_CB_NAMED_OPTION_SET			= 26,
 
 	/****** add new things just above ---^ ******/
 };
@@ -603,6 +682,7 @@ enum lws_token_indexes {
 	WSI_TOKEN_HTTP_URI_ARGS					= 76,
 	WSI_TOKEN_PROXY						= 77,
 	WSI_TOKEN_HTTP_X_REAL_IP				= 78,
+	WSI_TOKEN_HTTP1_0					= 79,
 
 	/****** add new things just above ---^ ******/
 
@@ -614,6 +694,7 @@ enum lws_token_indexes {
 	_WSI_TOKEN_CLIENT_URI,
 	_WSI_TOKEN_CLIENT_HOST,
 	_WSI_TOKEN_CLIENT_ORIGIN,
+	_WSI_TOKEN_CLIENT_METHOD,
 
 	/* always last real token index*/
 	WSI_TOKEN_COUNT,
@@ -748,6 +829,10 @@ enum http_status {
 	HTTP_STATUS_OK						= 200,
 	HTTP_STATUS_NO_CONTENT					= 204,
 
+	HTTP_STATUS_MOVED_PERMANENTLY				= 301,
+	HTTP_STATUS_FOUND					= 302,
+	HTTP_STATUS_SEE_OTHER					= 303,
+
 	HTTP_STATUS_BAD_REQUEST					= 400,
 	HTTP_STATUS_UNAUTHORIZED,
 	HTTP_STATUS_PAYMENT_REQUIRED,
@@ -1224,6 +1309,7 @@ struct lws_ext_options {
 };
 
 struct lws_ext_option_arg {
+	const char *option_name; /* may be NULL, option_index used then */
 	int option_index;
 	const char *start;
 	int len;
@@ -1246,117 +1332,213 @@ struct lws_extension {
 	 * This is part of the ABI, don't needlessly break compatibility */
 };
 
+
+#ifdef LWS_WITH_PLUGINS
+
+/* PLUGINS implies LIBUV */
+
+#define LWS_PLUGIN_API_MAGIC 180
+
+struct lws_plugin_capability {
+	unsigned int api_magic;	/* caller fills this in, plugin fills rest */
+	const struct lws_protocols *protocols;
+	int count_protocols;
+	const struct lws_extension *extensions;
+	int count_extensions;
+};
+
+typedef int (*lws_plugin_init_func)(struct lws_context *,
+				    struct lws_plugin_capability *);
+typedef int (*lws_plugin_destroy_func)(struct lws_context *);
+struct lws_plugin {
+	struct lws_plugin *list;
+#if (UV_VERSION_MAJOR > 0)
+	uv_lib_t lib;
+#else
+	void *l;
+#endif
+	char name[64];
+	struct lws_plugin_capability caps;
+};
+
+#endif
+
 /*
  * The internal exts are part of the public abi
  * If we add more extensions, publish the callback here  ------v
  */
 
-extern int lws_extension_callback_pm_deflate(
+LWS_EXTERN
+int lws_extension_callback_pm_deflate(
 	struct lws_context *context, const struct lws_extension *ext,
 	struct lws *wsi, enum lws_extension_callback_reasons reason,
 	void *user, void *in, size_t len);
 
+LWS_VISIBLE LWS_EXTERN int
+lws_set_extension_option(struct lws *wsi, const char *ext_name,
+			 const char *opt_name, const char *opt_val);
+
+struct lws_protocol_vhost_options {
+	const struct lws_protocol_vhost_options *next;
+	const struct lws_protocol_vhost_options *options;
+	const char *name;
+	const char *value;
+};
+
+struct lws_http_mount {
+	struct lws_http_mount *mount_next;
+	const char *mountpoint; /* mountpoint in http pathspace, eg, "/" */
+	const char *origin; /* path to be mounted, eg, "/var/www/warmcat.com" */
+	const char *def; /* default target, eg, "index.html" */
+
+	const struct lws_protocol_vhost_options *cgienv;
+	const struct lws_protocol_vhost_options *extra_mimetypes;
+
+	int cgi_timeout;
+	int cache_max_age;
+
+	unsigned int cache_reusable:1;
+	unsigned int cache_revalidate:1;
+	unsigned int cache_intermediaries:1;
+
+	unsigned char origin_protocol;
+	unsigned char mountpoint_len;
+};
 
 /**
  * struct lws_context_creation_info - parameters to create context with
  *
- * @port:	Port to listen on... you can use CONTEXT_PORT_NO_LISTEN to
+ * This is also used to create vhosts.... if LWS_SERVER_OPTION_EXPLICIT_VHOSTS
+ * is not given, then for backwards compatibility one vhost is created at
+ * context-creation time using the info from this struct.
+ *
+ * If LWS_SERVER_OPTION_EXPLICIT_VHOSTS is given, then no vhosts are created
+ * at the same time as the context, they are expected to be created afterwards.
+ *
+ * @port:	VHOST: Port to listen on... you can use CONTEXT_PORT_NO_LISTEN to
  *		suppress listening on any port, that's what you want if you are
  *		not running a websocket server at all but just using it as a
  *		client
- * @iface:	NULL to bind the listen socket to all interfaces, or the
+ * @iface:	VHOST: NULL to bind the listen socket to all interfaces, or the
  *		interface name, eg, "eth2"
- * @protocols:	Array of structures listing supported protocols and a protocol-
+ *		If options specifies LWS_SERVER_OPTION_UNIX_SOCK, this member is
+ *		the pathname of a UNIX domain socket. you can use the UNIX domain
+ *		sockets in abstract namespace, by prepending an @ symbole to the
+ *		socket name.
+ * @protocols:	VHOST: Array of structures listing supported protocols and a protocol-
  *		specific callback for each one.  The list is ended with an
  *		entry that has a NULL callback pointer.
  *		It's not const because we write the owning_server member
- * @extensions: NULL or array of lws_extension structs listing the
+ * @extensions: VHOST: NULL or array of lws_extension structs listing the
  *		extensions this context supports.  If you configured with
  *		--without-extensions, you should give NULL here.
- * @token_limits: NULL or struct lws_token_limits pointer which is initialized
+ * @token_limits: CONTEXT: NULL or struct lws_token_limits pointer which is initialized
  *		with a token length limit for each possible WSI_TOKEN_***
- * @ssl_cert_filepath:	If libwebsockets was compiled to use ssl, and you want
+ * @ssl_cert_filepath:	VHOST: If libwebsockets was compiled to use ssl, and you want
  *			to listen using SSL, set to the filepath to fetch the
  *			server cert from, otherwise NULL for unencrypted
- * @ssl_private_key_filepath: filepath to private key if wanting SSL mode;
+ * @ssl_private_key_filepath: VHOST: filepath to private key if wanting SSL mode;
  *			if this is set to NULL but sll_cert_filepath is set, the
  *			OPENSSL_CONTEXT_REQUIRES_PRIVATE_KEY callback is called
  *			to allow setting of the private key directly via openSSL
  *			library calls
- * @ssl_ca_filepath: CA certificate filepath or NULL
- * @ssl_cipher_list:	List of valid ciphers to use (eg,
+ * @ssl_ca_filepath: VHOST: CA certificate filepath or NULL
+ * @ssl_cipher_list:	VHOST: List of valid ciphers to use (eg,
  * 			"RC4-MD5:RC4-SHA:AES128-SHA:AES256-SHA:HIGH:!DSS:!aNULL"
  * 			or you can leave it as NULL to get "DEFAULT"
- * @http_proxy_address: If non-NULL, attempts to proxy via the given address.
+ * @http_proxy_address: VHOST: If non-NULL, attempts to proxy via the given address.
  *			If proxy auth is required, use format
  *			"username:password@server:port"
- * @http_proxy_port:	If http_proxy_address was non-NULL, uses this port at
+ * @http_proxy_port:	VHOST: If http_proxy_address was non-NULL, uses this port at
  * 			the address
- * @gid:	group id to change to after setting listen socket, or -1.
- * @uid:	user id to change to after setting listen socket, or -1.
- * @options:	0, or LWS_SERVER_OPTION_... bitfields
- * @user:	optional user pointer that can be recovered via the context
+ * @gid:	CONTEXT: group id to change to after setting listen socket, or -1.
+ * @uid:	CONTEXT: user id to change to after setting listen socket, or -1.
+ * @options:	VHOST + CONTEXT: 0, or LWS_SERVER_OPTION_... bitfields
+ * @user:	CONTEXT: optional user pointer that can be recovered via the context
  *		pointer using lws_context_user
- * @ka_time:	0 for no keepalive, otherwise apply this keepalive timeout to
+ * @ka_time:	CONTEXT: 0 for no keepalive, otherwise apply this keepalive timeout to
  *		all libwebsocket sockets, client or server
- * @ka_probes:	if ka_time was nonzero, after the timeout expires how many
+ * @ka_probes:	CONTEXT: if ka_time was nonzero, after the timeout expires how many
  *		times to try to get a response from the peer before giving up
  *		and killing the connection
- * @ka_interval: if ka_time was nonzero, how long to wait before each ka_probes
+ * @ka_interval: CONTEXT: if ka_time was nonzero, how long to wait before each ka_probes
  *		attempt
- * @provided_client_ssl_ctx: If non-null, swap out libwebsockets ssl
+ * @provided_client_ssl_ctx: CONTEXT: If non-null, swap out libwebsockets ssl
  *		implementation for the one provided by provided_ssl_ctx.
  *		Libwebsockets no longer is responsible for freeing the context
  *		if this option is selected.
- * @max_http_header_data: The max amount of header payload that can be handled
+ * @max_http_header_data: CONTEXT: The max amount of header payload that can be handled
  *		in an http request (unrecognized header payload is dropped)
- * @max_http_header_pool: The max number of connections with http headers that
+ * @max_http_header_pool: CONTEXT: The max number of connections with http headers that
  *		can be processed simultaneously (the corresponding memory is
  *		allocated for the lifetime of the context).  If the pool is
  *		busy new incoming connections must wait for accept until one
  *		becomes free.
- * @count_threads: how many contexts to create in an array, 0 = 1
- * @fd_limit_per_thread: nonzero means restrict each service thread to this
+ * @count_threads: CONTEXT: how many contexts to create in an array, 0 = 1
+ * @fd_limit_per_thread: CONTEXT: nonzero means restrict each service thread to this
  *		many fds, 0 means the default which is divide the process fd
  *		limit by the number of threads.
- * @timeout_secs: various processes involving network roundtrips in the
+ * @timeout_secs: VHOST: various processes involving network roundtrips in the
  *		library are protected from hanging forever by timeouts.  If
  *		nonzero, this member lets you set the timeout used in seconds.
  *		Otherwise a default timeout is used.
+ * @ecdh_curve: VHOST: if NULL, defaults to initializing server with "prime256v1"
+ * @vhost_name: VHOST: name of vhost, must match external DNS name used to
+ *		access the site, like "warmcat.com" as it's used to match
+ *		Host: header and / or SNI name for SSL.
+ * @plugin_dirs: CONTEXT: NULL, or NULL-terminated array of directories to
+ *		scan for lws protocol plugins at context creation time
+ * @pvo:	VHOST: pointer to optional linked list of per-vhost
+ *		options made accessible to protocols
+ * @keepalive_timeout: VHOST: (default = 0 = 60s) seconds to allow remote
+ *		client to hold on to an idle HTTP/1.1 connection
+ * @log_filepath: VHOST: filepath to append logs to... this is opened before
+ *		any dropping of initial privileges
+ * @mounts:	VHOST: optional linked list of mounts for this vhost
+ * @server_string: CONTEXT: string used in HTTP headers to identify server
+ *		software, if NULL, "libwebsockets".
  */
 
 struct lws_context_creation_info {
-	int port;
-	const char *iface;
-	const struct lws_protocols *protocols;
-	const struct lws_extension *extensions;
-	const struct lws_token_limits *token_limits;
-	const char *ssl_private_key_password;
-	const char *ssl_cert_filepath;
-	const char *ssl_private_key_filepath;
-	const char *ssl_ca_filepath;
-	const char *ssl_cipher_list;
-	const char *http_proxy_address;
-	unsigned int http_proxy_port;
-	int gid;
-	int uid;
-	unsigned int options;
-	void *user;
-	int ka_time;
-	int ka_probes;
-	int ka_interval;
+	int port;					/* VH */
+	const char *iface;				/* VH */
+	const struct lws_protocols *protocols;		/* VH */
+	const struct lws_extension *extensions;		/* VH */
+	const struct lws_token_limits *token_limits;	/* context */
+	const char *ssl_private_key_password;		/* VH */
+	const char *ssl_cert_filepath;			/* VH */
+	const char *ssl_private_key_filepath;		/* VH */
+	const char *ssl_ca_filepath;			/* VH */
+	const char *ssl_cipher_list;			/* VH */
+	const char *http_proxy_address;			/* VH */
+	unsigned int http_proxy_port;			/* VH */
+	int gid;					/* context */
+	int uid;					/* context */
+	unsigned int options;				/* VH + context */
+	void *user;					/* context */
+	int ka_time;					/* context */
+	int ka_probes;					/* context */
+	int ka_interval;				/* context */
 #ifdef LWS_OPENSSL_SUPPORT
-	SSL_CTX *provided_client_ssl_ctx;
+	SSL_CTX *provided_client_ssl_ctx;		/* context */
 #else /* maintain structure layout either way */
 	void *provided_client_ssl_ctx;
 #endif
 
-	short max_http_header_data;
-	short max_http_header_pool;
+	short max_http_header_data;			/* context */
+	short max_http_header_pool;			/* context */
 
-	unsigned int count_threads;
-	unsigned int fd_limit_per_thread;
-	unsigned int timeout_secs;
+	unsigned int count_threads;			/* context */
+	unsigned int fd_limit_per_thread;		/* context */
+	unsigned int timeout_secs;			/* VH */
+	const char *ecdh_curve;				/* VH */
+	const char *vhost_name;				/* VH */
+	const char * const *plugin_dirs;		/* context */
+	const struct lws_protocol_vhost_options *pvo;	/* VH */
+	int keepalive_timeout;				/* VH */
+	const char *log_filepath;			/* VH */
+	const struct lws_http_mount *mounts;		/* VH */
+	const char *server_string;			/* context */
 
 	/* Add new things just above here ---^
 	 * This is part of the ABI, don't needlessly break compatibility
@@ -1384,6 +1566,15 @@ struct lws_context_creation_info {
  * @ietf_version_or_minus_one: currently leave at 0 or -1
  * @userdata:	if non-NULL, use this as wsi user_data instead of malloc it
  * @client_exts: array of extensions that may be used on connection
+ * @method:	if non-NULL, do this http method instead of ws[s] upgrade.
+ *		use "GET" to be a simple http client connection
+ * @parent_wsi:	if another wsi is responsible for this connection, give it here.
+ *		this is used to make sure if the parent closes so do any
+ *		child connections first.
+ * @uri_replace_from: if non-NULL, when this string is found in URIs in
+ *		text/html content-encoding, it's replaced with @uri_replace_to
+ * @uri_replace_to: see above
+ * @vhost:	vhost to bind to (used to determine related SSL_CTX)
  */
 
 struct lws_client_connect_info {
@@ -1398,6 +1589,11 @@ struct lws_client_connect_info {
 	int ietf_version_or_minus_one;
 	void *userdata;
 	const struct lws_extension *client_exts;
+	const char *method;
+	struct lws *parent_wsi;
+	const char *uri_replace_from;
+	const char *uri_replace_to;
+	struct lws_vhost *vhost;
 
 	/* Add new things just above here ---^
 	 * This is part of the ABI, don't needlessly break compatibility
@@ -1410,6 +1606,21 @@ struct lws_client_connect_info {
 	void *_unused[4];
 };
 
+enum {
+	LWSMPRO_HTTP,
+	LWSMPRO_HTTPS,
+	LWSMPRO_FILE,
+	LWSMPRO_CGI,
+	LWSMPRO_REDIR_HTTP,
+	LWSMPRO_REDIR_HTTPS,
+};
+
+LWS_VISIBLE LWS_EXTERN int
+lws_json_dump_vhost(const struct lws_vhost *vh, char *buf, int len);
+
+LWS_VISIBLE LWS_EXTERN int
+lws_json_dump_context(const struct lws_context *context, char *buf, int len);
+
 LWS_VISIBLE LWS_EXTERN void
 lws_set_log_level(int level,
 		  void (*log_emit_function)(int level, const char *line));
@@ -1420,8 +1631,29 @@ lwsl_emit_syslog(int level, const char *line);
 LWS_VISIBLE LWS_EXTERN struct lws_context *
 lws_create_context(struct lws_context_creation_info *info);
 
+struct lws_vhost;
+
+LWS_EXTERN LWS_VISIBLE struct lws_vhost *
+lws_create_vhost(struct lws_context *context,
+		 struct lws_context_creation_info *info);
+
+LWS_VISIBLE LWS_EXTERN struct lws_vhost *
+lws_vhost_get(struct lws *wsi);
+
+LWS_VISIBLE LWS_EXTERN const struct lws_protocols *
+lws_protocol_get(struct lws *wsi);
+
+LWS_VISIBLE LWS_EXTERN void *
+lws_protocol_vh_priv_zalloc(struct lws_vhost *vhost, const struct lws_protocols *prot,
+			    int size);
+LWS_VISIBLE LWS_EXTERN void *
+lws_protocol_vh_priv_get(struct lws_vhost *vhost, const struct lws_protocols *prot);
+
 LWS_VISIBLE LWS_EXTERN int
-lws_set_proxy(struct lws_context *context, const char *proxy);
+lws_finalize_startup(struct lws_context *context);
+
+LWS_VISIBLE LWS_EXTERN int
+lws_set_proxy(struct lws_vhost *vhost, const char *proxy);
 
 LWS_VISIBLE LWS_EXTERN void
 lws_context_destroy(struct lws_context *context);
@@ -1445,26 +1677,32 @@ lws_interface_to_sa(int ipv6, const char *ifname, struct sockaddr_in *addr,
 LWS_VISIBLE LWS_EXTERN const unsigned char *
 lws_token_to_string(enum lws_token_indexes token);
 
-LWS_VISIBLE LWS_EXTERN int
+/* all the below must be LWS_WARN_UNUSED_RESULT as they can run short */
+
+LWS_VISIBLE LWS_EXTERN int LWS_WARN_UNUSED_RESULT
 lws_add_http_header_by_name(struct lws *wsi, const unsigned char *name,
 			    const unsigned char *value, int length,
 			    unsigned char **p, unsigned char *end);
-LWS_VISIBLE LWS_EXTERN int
+LWS_VISIBLE LWS_EXTERN int LWS_WARN_UNUSED_RESULT
 lws_finalize_http_header(struct lws *wsi, unsigned char **p,
 			 unsigned char *end);
-LWS_VISIBLE LWS_EXTERN int
+LWS_VISIBLE LWS_EXTERN int LWS_WARN_UNUSED_RESULT
 lws_add_http_header_by_token(struct lws *wsi, enum lws_token_indexes token,
 			     const unsigned char *value, int length,
 			     unsigned char **p, unsigned char *end);
-LWS_VISIBLE LWS_EXTERN int
+LWS_VISIBLE LWS_EXTERN int LWS_WARN_UNUSED_RESULT
 lws_add_http_header_content_length(struct lws *wsi,
 				   unsigned long content_length,
 				   unsigned char **p, unsigned char *end);
-LWS_VISIBLE LWS_EXTERN int
+LWS_VISIBLE LWS_EXTERN int LWS_WARN_UNUSED_RESULT
 lws_add_http_header_status(struct lws *wsi,
 			   unsigned int code, unsigned char **p,
 			   unsigned char *end);
 
+LWS_VISIBLE LWS_EXTERN int LWS_WARN_UNUSED_RESULT
+lws_http_redirect(struct lws *wsi, int code, const unsigned char *loc, int len,
+		  unsigned char **p, unsigned char *end);
+
 LWS_VISIBLE LWS_EXTERN int LWS_WARN_UNUSED_RESULT
 lws_http_transaction_completed(struct lws *wsi);
 
@@ -1478,31 +1716,29 @@ lws_ev_sigint_cfg(struct lws_context *context, int use_ev_sigint,
 LWS_VISIBLE LWS_EXTERN int
 lws_ev_initloop(struct lws_context *context, struct ev_loop *loop, int tsi);
 
-LWS_VISIBLE void
+LWS_VISIBLE LWS_EXTERN void
 lws_ev_sigint_cb(struct ev_loop *loop, struct ev_signal *watcher, int revents);
 #endif /* LWS_USE_LIBEV */
 
 #ifdef LWS_USE_LIBUV
-typedef void (lws_uv_signal_cb_t)(uv_loop_t *l, uv_signal_t *w, int revents);
-
 LWS_VISIBLE LWS_EXTERN int
 lws_uv_sigint_cfg(struct lws_context *context, int use_uv_sigint,
-		  lws_uv_signal_cb_t *cb);
+		  uv_signal_cb cb);
 
 LWS_VISIBLE LWS_EXTERN void
 lws_libuv_run(const struct lws_context *context, int tsi);
 
-LWS_VISIBLE void
+LWS_VISIBLE LWS_EXTERN void
 lws_libuv_stop(struct lws_context *context);
 
 LWS_VISIBLE LWS_EXTERN int
-lws_uv_initloop(struct lws_context *context, uv_loop_t *loop, uv_signal_cb cb, int tsi);
+lws_uv_initloop(struct lws_context *context, uv_loop_t *loop, int tsi);
 
 LWS_VISIBLE LWS_EXTERN uv_loop_t *
 lws_uv_getloop(struct lws_context *context, int tsi);
 
-LWS_VISIBLE void
-lws_uv_sigint_cb(uv_loop_t *loop, uv_signal_t *watcher, int revents);
+LWS_VISIBLE LWS_EXTERN void
+lws_uv_sigint_cb(uv_signal_t *watcher, int signum);
 #endif /* LWS_USE_LIBUV */
 
 LWS_VISIBLE LWS_EXTERN int
@@ -1537,6 +1773,8 @@ enum pending_timeout {
 	PENDING_TIMEOUT_AWAITING_CLIENT_HS_SEND			= 11,
 	PENDING_FLUSH_STORED_SEND_BEFORE_CLOSE			= 12,
 	PENDING_TIMEOUT_SHUTDOWN_FLUSH				= 13,
+	PENDING_TIMEOUT_CGI					= 14,
+	PENDING_TIMEOUT_HTTP_KEEPALIVE_IDLE			= 15,
 
 	/****** add new things just above ---^ ******/
 };
@@ -1597,7 +1835,7 @@ lws_set_timeout(struct lws *wsi, enum pending_timeout reason, int secs);
 #define u_int64_t unsigned long long
 #endif
 
-#if __x86_64__
+#if defined(__x86_64__)
 #define _LWS_PAD_SIZE 16	/* Intel recommended for best performance */
 #else
 #define _LWS_PAD_SIZE LWS_SIZEOFPTR   /* Size of a pointer on the target arch */
@@ -1653,10 +1891,18 @@ LWS_VISIBLE LWS_EXTERN int
 lws_callback_on_writable_all_protocol(const struct lws_context *context,
 				      const struct lws_protocols *protocol);
 
+LWS_VISIBLE LWS_EXTERN int
+lws_callback_on_writable_all_protocol_vhost(const struct lws_vhost *vhost,
+				      const struct lws_protocols *protocol);
+
 LWS_VISIBLE LWS_EXTERN int
 lws_callback_all_protocol(struct lws_context *context,
 			  const struct lws_protocols *protocol, int reason);
 
+LWS_VISIBLE LWS_EXTERN int
+lws_callback_all_protocol_vhost(struct lws_vhost *vh,
+			  const struct lws_protocols *protocol, int reason);
+
 LWS_VISIBLE LWS_EXTERN int
 lws_get_socket_fd(struct lws *wsi);
 
@@ -1714,6 +1960,9 @@ lws_client_connect_via_info(struct lws_client_connect_info * ccinfo);
 
 LWS_VISIBLE LWS_EXTERN struct lws *
 lws_adopt_socket(struct lws_context *context, lws_sockfd_type accept_fd);
+LWS_VISIBLE LWS_EXTERN struct lws *
+lws_adopt_socket_readbuf(struct lws_context *context, lws_sockfd_type accept_fd,
+		const char *readbuf, size_t len);
 
 LWS_VISIBLE LWS_EXTERN const char * LWS_WARN_UNUSED_RESULT
 lws_canonical_hostname(struct lws_context *context);
@@ -1740,6 +1989,10 @@ lws_frame_is_binary(struct lws *wsi);
 
 LWS_VISIBLE LWS_EXTERN int
 lws_is_ssl(struct lws *wsi);
+
+LWS_VISIBLE LWS_EXTERN int
+lws_is_cgi(struct lws *wsi);
+
 #ifdef LWS_SHA1_USE_OPENSSL_NAME
 #define lws_SHA1 SHA1
 #else
@@ -1812,6 +2065,52 @@ lws_get_context(const struct lws *wsi);
 LWS_VISIBLE LWS_EXTERN int LWS_WARN_UNUSED_RESULT
 lws_get_count_threads(struct lws_context *context);
 
+LWS_VISIBLE LWS_EXTERN struct lws * LWS_WARN_UNUSED_RESULT
+lws_get_parent(const struct lws *wsi);
+
+LWS_VISIBLE LWS_EXTERN struct lws * LWS_WARN_UNUSED_RESULT
+lws_get_child(const struct lws *wsi);
+
+#ifdef LWS_WITH_CGI
+enum lws_enum_stdinouterr {
+	LWS_STDIN = 0,
+	LWS_STDOUT = 1,
+	LWS_STDERR = 2,
+};
+
+enum lws_cgi_hdr_state {
+	LCHS_HEADER,
+	LCHS_CR1,
+	LCHS_LF1,
+	LCHS_CR2,
+	LCHS_LF2,
+	LHCS_PAYLOAD,
+	LCHS_SINGLE_0A,
+};
+
+struct lws_cgi_args {
+	struct lws **stdwsi; /* get fd with lws_get_socket_fd() */
+	enum lws_enum_stdinouterr ch;
+	unsigned char *data; /* for messages with payload */
+	enum lws_cgi_hdr_state hdr_state;
+	int len;
+};
+
+LWS_VISIBLE LWS_EXTERN int
+lws_cgi(struct lws *wsi, const char * const *exec_array,
+	int script_uri_path_len, int timeout_secs,
+	const struct lws_protocol_vhost_options *mp_cgienv);
+
+LWS_VISIBLE LWS_EXTERN int
+lws_cgi_write_split_stdout_headers(struct lws *wsi);
+
+LWS_VISIBLE LWS_EXTERN int
+lws_cgi_kill(struct lws *wsi);
+#endif
+
+LWS_VISIBLE LWS_EXTERN int
+lws_http_client_read(struct lws *wsi, char **buf, int *len);
+
 /*
  * Wsi-associated File Operations access helpers
  *
@@ -1891,6 +2190,20 @@ lws_ext_parse_options(const struct lws_extension *ext, struct lws *wsi,
 LWS_VISIBLE LWS_EXTERN void
 lws_set_allocator(void *(*realloc)(void *ptr, size_t size));
 
+/**
+ * lws_snprintf(): lws_snprintf that truncates the returned length too
+ *
+ * \param str: destination buffer
+ * \param size: bytes left in destination buffer
+ * \param format: format string
+ * \param ...: args for format
+ *
+ * This lets you correctly truncate buffers by concatenating lengths, if you
+ * reach the limit the reported length doesn't exceed the limit.
+ */
+LWS_VISIBLE LWS_EXTERN int
+lws_snprintf(char *str, size_t size, const char *format, ...);
+
 #ifdef __cplusplus
 }
 #endif
