@@ -96,7 +96,8 @@ int lws_issue_raw(struct lws *wsi, unsigned char *buf, size_t len)
 {
 	struct lws_context *context = lws_get_context(wsi);
 	size_t real_len = len;
-	int n, m;
+	unsigned int n;
+	int m;
 
 	if (!len)
 		return 0;
@@ -110,6 +111,8 @@ int lws_issue_raw(struct lws *wsi, unsigned char *buf, size_t len)
 		   wsi->trunc_offset))) {
 		lwsl_err("****** %x Sending new, pending truncated ...\n", wsi);
 		assert(0);
+
+		return -1;
 	}
 
 	m = lws_ext_cb_active(wsi, LWS_EXT_CB_PACKET_TX_DO_SEND, &buf, len);
@@ -123,11 +126,18 @@ int lws_issue_raw(struct lws *wsi, unsigned char *buf, size_t len)
 	if (!lws_socket_is_valid(wsi->sock))
 		lwsl_warn("** error invalid sock but expected to send\n");
 
+	/* limit sending */
+	n = wsi->protocol->rx_buffer_size;
+	if (!n)
+		n = LWS_MAX_SOCKET_IO_BUF;
+	n += LWS_PRE + 4;
+	if (n > len)
+		n = len;
+
 	/* nope, send it on the socket directly */
 	lws_latency_pre(context, wsi);
-	n = lws_ssl_capable_write(wsi, buf, len);
-	lws_latency(context, wsi, "send lws_issue_raw", n,
-		    (unsigned int)n == len);
+	n = lws_ssl_capable_write(wsi, buf, n);
+	lws_latency(context, wsi, "send lws_issue_raw", n, n == len);
 
 	switch (n) {
 	case LWS_SSL_CAPABLE_ERROR:
@@ -239,6 +249,12 @@ LWS_VISIBLE int lws_write(struct lws *wsi, unsigned char *buf, size_t len,
 	int pre = 0, n;
 	size_t orig_len = len;
 
+#ifdef LWS_WITH_ACCESS_LOG
+	wsi->access_log.sent += len;
+#endif
+	if (wsi->vhost)
+		wsi->vhost->tx += len;
+
 	if (wsi->state == LWSS_ESTABLISHED && wsi->u.ws.tx_draining_ext) {
 		/* remove us from the list */
 		struct lws **w = &pt->tx_draining_ext_list;
@@ -553,7 +569,7 @@ LWS_VISIBLE int lws_serve_http_file_fragment(struct lws *wsi)
 	unsigned long amount;
 	int n, m;
 
-	while (!lws_send_pipe_choked(wsi)) {
+	while (wsi->http2_substream || !lws_send_pipe_choked(wsi)) {
 		if (wsi->trunc_len) {
 			if (lws_issue_raw(wsi, wsi->trunc_alloc +
 					  wsi->trunc_offset,
@@ -593,7 +609,6 @@ LWS_VISIBLE int lws_serve_http_file_fragment(struct lws *wsi)
 all_sent:
 		if (!wsi->trunc_len && wsi->u.http.filepos == wsi->u.http.filelen) {
 			wsi->state = LWSS_HTTP;
-
 			/* we might be in keepalive, so close it off here */
 			lws_plat_file_close(wsi, wsi->u.http.fd);
 			wsi->u.http.fd = LWS_INVALID_FILE;
@@ -622,15 +637,18 @@ lws_ssl_capable_read_no_ssl(struct lws *wsi, unsigned char *buf, int len)
 	int n;
 
 	n = recv(wsi->sock, (char *)buf, len, 0);
-	if (n >= 0)
+	if (n >= 0) {
+		if (wsi->vhost)
+			wsi->vhost->rx += n;
 		return n;
+	}
 #if LWS_POSIX
 	if (LWS_ERRNO == LWS_EAGAIN ||
 	    LWS_ERRNO == LWS_EWOULDBLOCK ||
 	    LWS_ERRNO == LWS_EINTR)
 		return LWS_SSL_CAPABLE_MORE_SERVICE;
 #endif
-	lwsl_warn("error on reading from skt\n");
+	lwsl_notice("error on reading from skt : %d\n", LWS_ERRNO);
 	return LWS_SSL_CAPABLE_ERROR;
 }
 
