@@ -53,6 +53,8 @@ lws_handle_POLLOUT_event(struct lws *wsi, struct lws_pollfd *pollfd)
 #endif
 	int ret, m, n;
 
+	//lwsl_err("%s: %p\n", __func__, wsi);
+
 	/*
 	 * user callback is lowest priority to get these notifications
 	 * actually, since other pending things cannot be disordered
@@ -94,6 +96,12 @@ lws_handle_POLLOUT_event(struct lws *wsi, struct lws_pollfd *pollfd)
 		return 0; /* leave POLLOUT active */
 	}
 #endif
+
+#ifdef LWS_WITH_CGI
+	if (wsi->cgi)
+		goto user_service_go_again;
+#endif
+
 	/* Priority 3: pending control packets (pong or close)
 	 */
 	if ((wsi->state == LWSS_ESTABLISHED &&
@@ -143,6 +151,8 @@ lws_handle_POLLOUT_event(struct lws *wsi, struct lws_pollfd *pollfd)
 	/* Priority 6: user can get the callback
 	 */
 	m = lws_ext_cb_active(wsi, LWS_EXT_CB_IS_WRITEABLE, NULL, 0);
+	if (m)
+		return -1;
 #ifndef LWS_NO_EXTENSIONS
 	if (!wsi->extension_data_pending)
 		goto user_service;
@@ -237,6 +247,14 @@ user_service:
 			return 1;
 		}
 
+
+	if (!wsi->hdr_parsing_completed)
+		return 0;
+
+#ifdef LWS_WITH_CGI
+user_service_go_again:
+#endif
+
 #ifdef LWS_USE_HTTP2
 	/*
 	 * we are the 'network wsi' for potentially many muxed child wsi with
@@ -309,7 +327,9 @@ lws_service_timeout_check(struct lws *wsi, unsigned int sec)
 	 */
 	if ((time_t)sec > wsi->pending_timeout_limit) {
 #if LWS_POSIX
-		lwsl_notice("wsi %p: TIMEDOUT WAITING on %d (did hdr %d, ah %p, wl %d, pfd events %d)\n",
+		/* no need to log normal idle keepalive timeout */
+		if (wsi->pending_timeout != PENDING_TIMEOUT_HTTP_KEEPALIVE_IDLE)
+			lwsl_notice("wsi %p: TIMEDOUT WAITING on %d (did hdr %d, ah %p, wl %d, pfd events %d)\n",
 			    (void *)wsi, wsi->pending_timeout,
 			    wsi->hdr_parsing_completed, wsi->u.hdr.ah,
 			    pt->ah_wait_list_length,
@@ -323,6 +343,11 @@ lws_service_timeout_check(struct lws *wsi, unsigned int sec)
 		 * cleanup like flush partials.
 		 */
 		wsi->socket_is_permanently_unusable = 1;
+		if (wsi->mode == LWSCM_WSCL_WAITING_SSL)
+			wsi->vhost->protocols[0].callback(wsi,
+				LWS_CALLBACK_CLIENT_CONNECTION_ERROR,
+				wsi->user_space, NULL, 0);
+
 		lws_close_free_wsi(wsi, LWS_CLOSE_STATUS_NOSTATUS);
 
 		return 1;
@@ -343,6 +368,8 @@ int lws_rxflow_cache(struct lws *wsi, unsigned char *buf, int n, int len)
 	/* a new rxflow, buffer it and warn caller */
 	lwsl_info("new rxflow input buffer len %d\n", len - n);
 	wsi->rxflow_buffer = lws_malloc(len - n);
+	if (!wsi->rxflow_buffer)
+		return -1;
 	wsi->rxflow_len = len - n;
 	wsi->rxflow_pos = 0;
 	memcpy(wsi->rxflow_buffer, buf + n, len - n);
@@ -367,13 +394,13 @@ lws_service_adjust_timeout(struct lws_context *context, int timeout_ms, int tsi)
 
 	/* 1) if we know we are draining rx ext, do not wait in poll */
 	if (pt->rx_draining_ext_list)
-		timeout_ms = 0;
+		return 0;
 
 #ifdef LWS_OPENSSL_SUPPORT
 	/* 2) if we know we have non-network pending data, do not wait in poll */
 	if (lws_ssl_anybody_has_buffered_read_tsi(context, tsi)) {
-		timeout_ms = 0;
-		lwsl_err("ssl buffered read\n");
+		lwsl_info("ssl buffered read\n");
+		return 0;
 	}
 #endif
 
@@ -381,9 +408,11 @@ lws_service_adjust_timeout(struct lws_context *context, int timeout_ms, int tsi)
 	for (n = 0; n < context->max_http_header_pool; n++)
 		if (pt->ah_pool[n].rxpos != pt->ah_pool[n].rxlen) {
 			/* any ah with pending rx must be attached to someone */
-			assert(pt->ah_pool[n].wsi);
-			timeout_ms = 0;
-			break;
+			if (!pt->ah_pool[n].wsi) {
+				lwsl_err("%s: assert: no wsi attached to ah\n", __func__);
+				assert(0);
+			}
+			return 0;
 		}
 
 	return timeout_ms;
@@ -468,6 +497,136 @@ lws_service_flag_pending(struct lws_context *context, int tsi)
 	return forced;
 }
 
+#ifndef LWS_NO_CLIENT
+
+LWS_VISIBLE int
+lws_http_client_read(struct lws *wsi, char **buf, int *len)
+{
+	int rlen, n;
+
+
+
+	rlen = lws_ssl_capable_read(wsi, (unsigned char *)*buf, *len);
+	if (rlen < 0)
+		return -1;
+
+	*len = rlen;
+	if (rlen == 0)
+		return 0;
+
+//	lwsl_err("%s: read %d\n", __func__, rlen);
+
+	/* allow the source to signal he has data again next time */
+	wsi->client_rx_avail = 0;
+	lws_change_pollfd(wsi, 0, LWS_POLLIN);
+
+	/*
+	 * server may insist on transfer-encoding: chunked,
+	 * so http client must deal with it
+	 */
+spin_chunks:
+	while (wsi->chunked && (wsi->chunk_parser != ELCP_CONTENT) && *len) {
+		switch (wsi->chunk_parser) {
+		case ELCP_HEX:
+			if ((*buf)[0] == '\x0d') {
+				wsi->chunk_parser = ELCP_CR;
+				break;
+			}
+			n = char_to_hex((*buf)[0]);
+			if (n < 0)
+				return -1;
+			wsi->chunk_remaining <<= 4;
+			wsi->chunk_remaining |= n;
+			break;
+		case ELCP_CR:
+			if ((*buf)[0] != '\x0a')
+				return -1;
+			wsi->chunk_parser = ELCP_CONTENT;
+			lwsl_info("chunk %d\n", wsi->chunk_remaining);
+			if (wsi->chunk_remaining)
+				break;
+			lwsl_info("final chunk\n");
+			goto completed;
+
+		case ELCP_CONTENT:
+			break;
+
+		case ELCP_POST_CR:
+			if ((*buf)[0] != '\x0d')
+				return -1;
+
+			wsi->chunk_parser = ELCP_POST_LF;
+			break;
+
+		case ELCP_POST_LF:
+			if ((*buf)[0] != '\x0a')
+				return -1;
+
+			wsi->chunk_parser = ELCP_HEX;
+			wsi->chunk_remaining = 0;
+			break;
+		}
+		(*buf)++;
+		(*len)--;
+	}
+
+	if (wsi->chunked && !wsi->chunk_remaining)
+		return 0;
+
+	if (wsi->u.http.content_remain &&
+	    (int)wsi->u.http.content_remain < *len)
+		n = wsi->u.http.content_remain;
+	else
+		n = *len;
+
+	if (wsi->chunked && wsi->chunk_remaining &&
+	    wsi->chunk_remaining < n)
+		n = wsi->chunk_remaining;
+
+#ifdef LWS_WITH_HTTP_PROXY
+	/* hubbub */
+	if (wsi->perform_rewrite)
+		lws_rewrite_parse(wsi->rw, (unsigned char *)*buf, n);
+	else
+#endif
+		if (user_callback_handle_rxflow(wsi->protocol->callback,
+				wsi, LWS_CALLBACK_RECEIVE_CLIENT_HTTP_READ,
+				wsi->user_space, *buf, n))
+			return -1;
+
+	if (wsi->chunked && wsi->chunk_remaining) {
+		(*buf) += n;
+		wsi->chunk_remaining -= n;
+		*len -= n;
+	}
+
+	if (wsi->chunked && !wsi->chunk_remaining)
+		wsi->chunk_parser = ELCP_POST_CR;
+
+	if (wsi->chunked && *len) {
+		goto spin_chunks;
+	}
+
+	if (wsi->chunked)
+		return 0;
+
+	wsi->u.http.content_remain -= n;
+	if (wsi->u.http.content_remain || !wsi->u.http.content_length)
+		return 0;
+
+completed:
+	if (user_callback_handle_rxflow(wsi->protocol->callback,
+			wsi, LWS_CALLBACK_COMPLETED_CLIENT_HTTP,
+			wsi->user_space, NULL, 0))
+		return -1;
+
+	if (lws_http_transaction_completed_client(wsi))
+		return -1;
+
+	return 0;
+}
+#endif
+
 /**
  * lws_service_fd() - Service polled socket with something waiting
  * @context:	Websocket context
@@ -502,9 +661,12 @@ lws_service_fd_tsi(struct lws_context *context, struct lws_pollfd *pollfd, int t
 	char draining_flow = 0;
 	int timed_out = 0;
 	time_t now;
-	int n, m;
+	int n = 0, m;
 	int more;
 
+	if (!context->protocol_init_done)
+		lws_protocol_init(context);
+
 	/*
 	 * you can call us with pollfd = NULL to just allow the once-per-second
 	 * global timeout checks; if less than a second since the last check
@@ -538,6 +700,9 @@ lws_service_fd_tsi(struct lws_context *context, struct lws_pollfd *pollfd, int t
 			}
 			wsi = wsi1;
 		}
+#ifdef LWS_WITH_CGI
+		lws_cgi_kill_terminated(pt);
+#endif
 #if 0
 		{
 			char s[300], *p = s;
@@ -572,7 +737,6 @@ lws_service_fd_tsi(struct lws_context *context, struct lws_pollfd *pollfd, int t
 	 */
 
 #if LWS_POSIX
-
 	/* handle session socket closed */
 
 	if ((!(pollfd->revents & pollfd->events & LWS_POLLIN)) &&
@@ -591,24 +755,36 @@ lws_service_fd_tsi(struct lws_context *context, struct lws_pollfd *pollfd, int t
 
 #endif
 
+	lwsl_debug("fd=%d, revents=%d\n", pollfd->fd, pollfd->revents);
+
 	/* okay, what we came here to do... */
 
 	switch (wsi->mode) {
 	case LWSCM_HTTP_SERVING:
+	case LWSCM_HTTP_CLIENT:
 	case LWSCM_HTTP_SERVING_ACCEPTED:
 	case LWSCM_SERVER_LISTENER:
 	case LWSCM_SSL_ACK_PENDING:
+		if (wsi->state == LWSS_CLIENT_HTTP_ESTABLISHED)
+			goto handled;
+
+#ifdef LWS_WITH_CGI
+		if (wsi->cgi && (pollfd->revents & LWS_POLLOUT)) {
+			n = lws_handle_POLLOUT_event(wsi, pollfd);
+			if (n)
+				goto close_and_handled;
+			goto handled;
+		}
+#endif
 		n = lws_server_socket_service(context, wsi, pollfd);
 		if (n) /* closed by above */
 			return 1;
-		pending = lws_ssl_pending(wsi);
-		if (pending)
-			goto handle_pending;
 		goto handled;
 
 	case LWSCM_WS_SERVING:
 	case LWSCM_WS_CLIENT:
 	case LWSCM_HTTP2_SERVING:
+	case LWSCM_HTTP_CLIENT_ACCEPTED:
 
 		/* 1: something requested a callback when it was OK to write */
 
@@ -636,7 +812,7 @@ lws_service_fd_tsi(struct lws_context *context, struct lws_pollfd *pollfd, int t
 			wsi->u.ws.tx_draining_ext = 0;
 		}
 
-		if (wsi->u.ws.tx_draining_ext) {
+		if (wsi->u.ws.tx_draining_ext)
 			/* we cannot deal with new RX until the TX ext
 			 * path has been drained.  It's because new
 			 * rx will, eg, crap on the wsi rx buf that
@@ -646,7 +822,6 @@ lws_service_fd_tsi(struct lws_context *context, struct lws_pollfd *pollfd, int t
 			 * to avoid blocking.
 			 */
 			break;
-		}
 
 		if (!(wsi->rxflow_change_to & LWS_RXFLOW_ALLOW))
 			/* We cannot deal with any kind of new RX
@@ -703,36 +878,60 @@ lws_service_fd_tsi(struct lws_context *context, struct lws_pollfd *pollfd, int t
 
 		if (!(pollfd->revents & pollfd->events & LWS_POLLIN))
 			break;
-read:
 
+read:
 		/* all the union members start with hdr, so even in ws mode
 		 * we can deal with the ah via u.hdr
 		 */
 		if (wsi->u.hdr.ah) {
-			lwsl_err("%s: %p: using inherited ah rx\n", __func__, wsi);
+			lwsl_info("%s: %p: inherited ah rx\n", __func__, wsi);
 			eff_buf.token_len = wsi->u.hdr.ah->rxlen -
 					    wsi->u.hdr.ah->rxpos;
 			eff_buf.token = (char *)wsi->u.hdr.ah->rx +
 					wsi->u.hdr.ah->rxpos;
 		} else {
+			if (wsi->mode != LWSCM_HTTP_CLIENT_ACCEPTED) {
+				eff_buf.token_len = lws_ssl_capable_read(wsi,
+					pt->serv_buf, pending ? pending :
+							LWS_MAX_SOCKET_IO_BUF);
+				switch (eff_buf.token_len) {
+				case 0:
+					lwsl_info("%s: zero length read\n", __func__);
+					goto close_and_handled;
+				case LWS_SSL_CAPABLE_MORE_SERVICE:
+					lwsl_info("SSL Capable more service\n");
+					n = 0;
+					goto handled;
+				case LWS_SSL_CAPABLE_ERROR:
+					lwsl_info("Closing when error\n");
+					goto close_and_handled;
+				}
 
-			eff_buf.token_len = lws_ssl_capable_read(wsi, pt->serv_buf,
-					     pending ? pending : LWS_MAX_SOCKET_IO_BUF);
-			switch (eff_buf.token_len) {
-			case 0:
-				lwsl_info("service_fd: closing due to 0 length read\n");
-				goto close_and_handled;
-			case LWS_SSL_CAPABLE_MORE_SERVICE:
-				lwsl_info("SSL Capable more service\n");
-				n = 0;
-				goto handled;
-			case LWS_SSL_CAPABLE_ERROR:
-				lwsl_info("Closing when error\n");
-				goto close_and_handled;
+				eff_buf.token = (char *)pt->serv_buf;
 			}
-
-			eff_buf.token = (char *)pt->serv_buf;
 		}
+
+drain:
+#ifndef LWS_NO_CLIENT
+		if (wsi->mode == LWSCM_HTTP_CLIENT_ACCEPTED) {
+
+			/*
+			 * simply mark ourselves as having readable data
+			 * and turn off our POLLIN
+			 */
+			wsi->client_rx_avail = 1;
+			lws_change_pollfd(wsi, LWS_POLLIN, 0);
+
+			/* let user code know, he'll usually ask for writeable
+			 * callback and drain / reenable it there
+			 */
+			if (user_callback_handle_rxflow(
+					wsi->protocol->callback,
+					wsi, LWS_CALLBACK_RECEIVE_CLIENT_HTTP,
+					wsi->user_space, NULL, 0))
+				goto close_and_handled;
+		}
+#endif
 		/*
 		 * give any active extensions a chance to munge the buffer
 		 * before parse.  We pass in a pointer to an lws_tokens struct
@@ -744,7 +943,6 @@ read:
 		 * extension callback handling, just the normal input buffer is
 		 * used then so it is efficient.
 		 */
-drain:
 		do {
 			more = 0;
 
@@ -779,18 +977,18 @@ drain:
 		} while (more);
 
 		if (wsi->u.hdr.ah) {
-			lwsl_err("%s: %p: detaching inherited used ah\n", __func__, wsi);
+			lwsl_info("%s: %p: detaching inherited used ah\n",
+				 __func__, wsi);
 			/* show we used all the pending rx up */
 			wsi->u.hdr.ah->rxpos = wsi->u.hdr.ah->rxlen;
 			/* we can run the normal ah detach flow despite
 			 * being in ws union mode, since all union members
 			 * start with hdr */
-			lws_header_table_detach(wsi);
+			lws_header_table_detach(wsi, 0);
 		}
 
 		pending = lws_ssl_pending(wsi);
 		if (pending) {
-handle_pending:
 			pending = pending > LWS_MAX_SOCKET_IO_BUF ?
 					LWS_MAX_SOCKET_IO_BUF : pending;
 			goto read;
@@ -809,7 +1007,43 @@ handle_pending:
 		}
 
 		break;
+#ifdef LWS_WITH_CGI
+	case LWSCM_CGI: /* we exist to handle a cgi's stdin/out/err data...
+			 * do the callback on our master wsi
+			 */
+		{
+			struct lws_cgi_args args;
 
+			if (wsi->cgi_channel >= LWS_STDOUT &&
+			    !(pollfd->revents & pollfd->events & LWS_POLLIN))
+				break;
+			if (wsi->cgi_channel == LWS_STDIN &&
+			    !(pollfd->revents & pollfd->events & LWS_POLLOUT))
+				break;
+
+			if (wsi->cgi_channel == LWS_STDIN)
+				if (lws_change_pollfd(wsi, LWS_POLLOUT, 0)) {
+					lwsl_info("failed at set pollfd\n");
+					return 1;
+				}
+
+			args.ch = wsi->cgi_channel;
+			args.stdwsi = &wsi->parent->cgi->stdwsi[0];
+			args.hdr_state = wsi->hdr_state;
+
+			//lwsl_err("CGI LWS_STDOUT waiting wsi %p mode %d state %d\n",
+			//	 wsi->parent, wsi->parent->mode, wsi->parent->state);
+
+			if (user_callback_handle_rxflow(
+					wsi->parent->protocol->callback,
+					wsi->parent, LWS_CALLBACK_CGI,
+					wsi->parent->user_space,
+					(void *)&args, 0))
+				return 1;
+
+			break;
+		}
+#endif
 	default:
 #ifdef LWS_NO_CLIENT
 		break;
