@@ -62,6 +62,10 @@
 #include <sys/socket.h>
 #endif
 
+#if defined(__NetBSD__)
+#include <netinet/in.h>
+#endif
+
 #define MAX_FUZZ_BUF (1024 * 1024)
 
 enum types {
@@ -953,4 +957,3 @@ bail1:
 
 	return 0;
 }
-
