@@ -18,6 +18,8 @@
  * Public Domain.
  */
 
+#include "lws_config.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <getopt.h>
@@ -39,6 +41,15 @@ static int deny_deflate, deny_mux, longlived, mirror_lifetime;
 static struct lws *wsi_dumb, *wsi_mirror;
 static volatile int force_exit;
 static unsigned int opts;
+#if defined(LWS_USE_POLARSSL)
+#else
+#if defined(LWS_USE_MBEDTLS)
+#else
+#if defined(LWS_OPENSSL_SUPPORT) && defined(LWS_HAVE_SSL_CTX_set1_param)
+char crl_path[1024] = "";
+#endif
+#endif
+#endif
 
 /*
  * This demo shows how to connect multiple websockets simultaneously to a
@@ -114,6 +125,60 @@ callback_dumb_increment(struct lws *wsi, enum lws_callback_reasons reason,
 			return 1;
 		break;
 
+	case LWS_CALLBACK_RECEIVE_CLIENT_HTTP:
+		{
+			char buffer[1024 + LWS_PRE];
+			char *px = buffer + LWS_PRE;
+			int lenx = sizeof(buffer) - LWS_PRE;
+
+			lwsl_notice("LWS_CALLBACK_RECEIVE_CLIENT_HTTP\n");
+
+			/*
+			 * Often you need to flow control this by something
+			 * else being writable.  In that case call the api
+			 * to get a callback when writable here, and do the
+			 * pending client read in the writeable callback of
+			 * the output.
+			 */
+			if (lws_http_client_read(wsi, &px, &lenx) < 0)
+				return -1;
+			while (lenx--)
+				putchar(*px++);
+		}
+		break;
+
+	case LWS_CALLBACK_COMPLETED_CLIENT_HTTP:
+		wsi_dumb = NULL;
+		force_exit = 1;
+		break;
+
+#if defined(LWS_USE_POLARSSL)
+#else
+#if defined(LWS_USE_MBEDTLS)
+#else
+#if defined(LWS_OPENSSL_SUPPORT) && defined(LWS_HAVE_SSL_CTX_set1_param)
+	case LWS_CALLBACK_OPENSSL_LOAD_EXTRA_CLIENT_VERIFY_CERTS:
+		if (crl_path[0]) {
+			/* Enable CRL checking of the server certificate */
+			X509_VERIFY_PARAM *param = X509_VERIFY_PARAM_new();
+			X509_VERIFY_PARAM_set_flags(param, X509_V_FLAG_CRL_CHECK);
+			SSL_CTX_set1_param((SSL_CTX*)user, param);
+			X509_STORE *store = SSL_CTX_get_cert_store((SSL_CTX*)user);
+			X509_LOOKUP *lookup = X509_STORE_add_lookup(store, X509_LOOKUP_file());
+			int n = X509_load_cert_crl_file(lookup, crl_path, X509_FILETYPE_PEM);
+			X509_VERIFY_PARAM_free(param);
+			if (n != 1) {
+				char errbuf[256];
+				n = ERR_get_error();
+				lwsl_err("LWS_CALLBACK_OPENSSL_LOAD_EXTRA_CLIENT_VERIFY_CERTS: SSL error: %s (%d)\n", ERR_error_string(n, errbuf), n);
+				return 1;
+			}
+		}
+		break;
+#endif
+#endif
+#endif
+
 	default:
 		break;
 	}
@@ -250,6 +315,12 @@ static struct option options[] = {
 	{ "undeflated",	no_argument,		NULL, 'u' },
 	{ "nomux",	no_argument,		NULL, 'n' },
 	{ "longlived",	no_argument,		NULL, 'l' },
+	{ "ssl-cert",  required_argument,	NULL, 'C' },
+	{ "ssl-key",  required_argument,	NULL, 'K' },
+	{ "ssl-ca",  required_argument,		NULL, 'A' },
+#if defined(LWS_OPENSSL_SUPPORT) && defined(LWS_HAVE_SSL_CTX_set1_param)
+	{ "ssl-crl",  required_argument,		NULL, 'R' },
+#endif
 	{ NULL, 0, 0, 0 }
 };
 
@@ -269,12 +340,15 @@ static int ratelimit_connects(unsigned int *last, unsigned int secs)
 int main(int argc, char **argv)
 {
 	int n = 0, ret = 0, port = 7681, use_ssl = 0, ietf_version = -1;
-	unsigned int rl_dumb = 0, rl_mirror = 0;
+	unsigned int rl_dumb = 0, rl_mirror = 0, do_ws = 1;
 	struct lws_context_creation_info info;
 	struct lws_client_connect_info i;
 	struct lws_context *context;
 	const char *prot, *p;
 	char path[300];
+	char cert_path[1024] = "";
+	char key_path[1024] = "";
+	char ca_path[1024] = "";
 
 	memset(&info, 0, sizeof info);
 
@@ -285,7 +359,7 @@ int main(int argc, char **argv)
 		goto usage;
 
 	while (n >= 0) {
-		n = getopt_long(argc, argv, "nuv:hsp:d:l", options, NULL);
+		n = getopt_long(argc, argv, "nuv:hsp:d:lC:K:A:", options, NULL);
 		if (n < 0)
 			continue;
 		switch (n) {
@@ -310,6 +384,30 @@ int main(int argc, char **argv)
 		case 'n':
 			deny_mux = 1;
 			break;
+		case 'C':
+			strncpy(cert_path, optarg, sizeof(cert_path) - 1);
+			cert_path[sizeof(cert_path) - 1] = '\0';
+			break;
+		case 'K':
+			strncpy(key_path, optarg, sizeof(key_path) - 1);
+			key_path[sizeof(key_path) - 1] = '\0';
+			break;
+		case 'A':
+			strncpy(ca_path, optarg, sizeof(ca_path) - 1);
+			ca_path[sizeof(ca_path) - 1] = '\0';
+			break;
+#if defined(LWS_USE_POLARSSL)
+#else
+#if defined(LWS_USE_MBEDTLS)
+#else
+#if defined(LWS_OPENSSL_SUPPORT) && defined(LWS_HAVE_SSL_CTX_set1_param)
+		case 'R':
+			strncpy(crl_path, optarg, sizeof(crl_path) - 1);
+			crl_path[sizeof(crl_path) - 1] = '\0';
+			break;
+#endif
+#endif
+#endif
 		case 'h':
 			goto usage;
 		}
@@ -335,7 +433,8 @@ int main(int argc, char **argv)
 	if (!strcmp(prot, "http") || !strcmp(prot, "ws"))
 		use_ssl = 0;
 	if (!strcmp(prot, "https") || !strcmp(prot, "wss"))
-		use_ssl = 1;
+		if (!use_ssl)
+			use_ssl = 1;
 
 	/*
 	 * create the websockets context.  This tracks open connections and
@@ -350,6 +449,41 @@ int main(int argc, char **argv)
 	info.gid = -1;
 	info.uid = -1;
 
+	if (use_ssl) {
+		info.options |= LWS_SERVER_OPTION_DO_SSL_GLOBAL_INIT;
+
+		/*
+		 * If the server wants us to present a valid SSL client certificate
+		 * then we can set it up here.
+		 */
+
+		if (cert_path[0])
+			info.ssl_cert_filepath = cert_path;
+		if (key_path[0])
+			info.ssl_private_key_filepath = key_path;
+
+		/*
+		 * A CA cert and CRL can be used to validate the cert send by the server
+		 */
+		if (ca_path[0])
+			info.ssl_ca_filepath = ca_path;
+#if defined(LWS_USE_POLARSSL)
+#else
+#if defined(LWS_USE_MBEDTLS)
+#else
+#if defined(LWS_OPENSSL_SUPPORT) && defined(LWS_HAVE_SSL_CTX_set1_param)
+		else if (crl_path[0])
+			lwsl_notice("WARNING, providing a CRL requires a CA cert!\n");
+#endif
+#endif
+#endif
+
+		if (use_ssl == 1)
+			lwsl_notice(" Cert must validate correctly (use -s to allow selfsigned)\n");
+		else
+			lwsl_notice(" Selfsigned certs allowed\n");
+	}
+
 	context = lws_create_context(&info);
 	if (context == NULL) {
 		fprintf(stderr, "Creating libwebsocket context failed\n");
@@ -362,6 +496,14 @@ int main(int argc, char **argv)
 	i.origin = i.address;
 	i.ietf_version_or_minus_one = ietf_version;
 	i.client_exts = exts;
+
+	if (!strcmp(prot, "http") || !strcmp(prot, "https")) {
+		lwsl_notice("using %s mode (non-ws)\n", prot);
+		i.method = "GET";
+		do_ws = 0;
+	} else
+		lwsl_notice("using %s mode (ws)\n", prot);
+
 	/*
 	 * sit there servicing the websocket context to handle incoming
 	 * packets, and drawing random circles on the mirror protocol websocket
@@ -374,17 +516,23 @@ int main(int argc, char **argv)
 
 	while (!force_exit) {
 
-		if (!wsi_dumb && ratelimit_connects(&rl_dumb, 2u)) {
-			lwsl_notice("dumb: connecting\n");
-			i.protocol = protocols[PROTOCOL_DUMB_INCREMENT].name;
-			wsi_dumb = lws_client_connect_via_info(&i);
-		}
+		if (do_ws) {
+			if (!wsi_dumb && ratelimit_connects(&rl_dumb, 2u)) {
+				lwsl_notice("dumb: connecting\n");
+				i.protocol = protocols[PROTOCOL_DUMB_INCREMENT].name;
+				wsi_dumb = lws_client_connect_via_info(&i);
+			}
 
-		if (!wsi_mirror && ratelimit_connects(&rl_mirror, 2u)) {
-			lwsl_notice("mirror: connecting\n");
-			i.protocol = protocols[PROTOCOL_LWS_MIRROR].name;
-			wsi_mirror = lws_client_connect_via_info(&i);
-		}
+			if (!wsi_mirror && ratelimit_connects(&rl_mirror, 2u)) {
+				lwsl_notice("mirror: connecting\n");
+				i.protocol = protocols[PROTOCOL_LWS_MIRROR].name;
+				wsi_mirror = lws_client_connect_via_info(&i);
+			}
+		} else
+			if (!wsi_dumb && ratelimit_connects(&rl_dumb, 2u)) {
+				lwsl_notice("http: connecting\n");
+				wsi_dumb = lws_client_connect_via_info(&i);
+			}
 
 		lws_service(context, 500);
 	}
