@@ -7,6 +7,10 @@
   #define DELTA_EPOCH_IN_MICROSECS  11644473600000000ULL
 #endif
 
+#ifdef LWS_MINGW_SUPPORT
+  #include <winsock2.h>
+#endif
+
 #ifndef _TIMEZONE_DEFINED 
 struct timezone 
 {
